pub struct MeshComponent {
pub key: String,
}Expand description
Select a mesh for a Renderable by string key.
This is intended to be attached as a descendant of a RenderableComponent.
The key can refer to imported meshes (e.g. “{gltf}:{mesh}:{prim}”) registered in RenderAssets.
Fields
key: StringImplementations