pub enum TextureSource {
Uri(String),
Handle(TextureHandle),
}Expand description
Where a texture comes from.
For v1 GLTF support, imported textures can be represented as a namespaced URI string like “{gltf_name}:{image_name_or_index}”.
Variants
Uri(String)
A URI-like string. Today this is typically a filesystem path (optionally file://),
but it can also be a virtual key (e.g. for imported textures).
Handle(TextureHandle)
A renderer-provided handle, already uploaded.