pub struct UVComponent {
pub uvs: Vec<[f32; 2]>,
}Expand description
Per-vertex UVs for a renderable.
This is intended to be attached as a descendant of a RenderableComponent.
Lifecycle note:
- UV overrides are applied when the renderable is flushed into
VisualWorld/ uploaded. - If fewer UVs are provided than the mesh’s vertex count, the missing UVs are filled with 0.
Fields
uvs: Vec<[f32; 2]>Implementations
Sourceimpl UVComponent
impl UVComponent