pub struct RxIntentExecutor;Expand description
Built-in executor for intent signals.
This is intentionally minimal scaffolding for the ongoing refactor described in:
- docs/signals.md
The goal is to keep handlers observers-only, and execute side effects via intent signals.
Implementations
Sourceimpl RxIntentExecutor
impl RxIntentExecutor
pub fn new() -> Self
Sourcepub fn handles_value(value: &IntentValue) -> bool
pub fn handles_value(value: &IntentValue) -> bool
Returns whether this executor is expected to handle the given signal value.
Note: during migration, this is intentionally conservative; it is not yet wired into the drain loop.
Sourcepub fn execute(
&mut self,
world: &mut World,
render_assets: &mut RenderAssets,
emit: &mut dyn SignalEmitter,
env: &Signal,
)
pub fn execute( &mut self, world: &mut World, render_assets: &mut RenderAssets, emit: &mut dyn SignalEmitter, env: &Signal, )
Execute an intent signal, emitting follow-up mutation signals via emit.