pub struct SignalPipelineProcessor;Expand description
Pre-execution processor for intent signals.
This is intended to become the hook point for intent routing/forwarding based on per-target “pipeline” components in the ECS topology.
Current behavior: no-op passthrough.
Implementations
Sourceimpl SignalPipelineProcessor
impl SignalPipelineProcessor
pub fn new() -> Self
Sourcepub fn process_intent(
&mut self,
world: &World,
rx: &RxWorld,
env: Signal,
) -> Signal
pub fn process_intent( &mut self, world: &World, rx: &RxWorld, env: Signal, ) -> Signal
Process an intent-containing signal envelope before it is executed.
This is a no-op for now (returns env unchanged).
Sourcepub fn recipient_component_ids(value: &IntentValue) -> Option<&[ComponentId]>
pub fn recipient_component_ids(value: &IntentValue) -> Option<&[ComponentId]>
Returns the standardized intent recipients, if the intent variant has them.