pub fn spawn_decode_thread(
complete_tx: Sender<LoadedClipMessage>,
) -> DecodeThreadHandleExpand description
Spawn the worker. Completion messages go to complete_tx (main
thread). The main thread forwards them to the audio RT thread.
pub fn spawn_decode_thread(
complete_tx: Sender<LoadedClipMessage>,
) -> DecodeThreadHandleSpawn the worker. Completion messages go to complete_tx (main
thread). The main thread forwards them to the audio RT thread.