pub struct BoundsSystem;Expand description
Utility system for calculating aggregate bounds of component subtrees.
Implementations
Sourceimpl BoundsSystem
impl BoundsSystem
pub fn new() -> Self
Sourcepub fn calculate_subtree_local_bounds(
world: &World,
render_assets: &RenderAssets,
root: ComponentId,
) -> Option<Aabb>
pub fn calculate_subtree_local_bounds( world: &World, render_assets: &RenderAssets, root: ComponentId, ) -> Option<Aabb>
Recursively compute the aggregate AABB of a component subtree in the root’s coordinate frame.
This walks the child topology and unions the transformed AABBs of all
RenderableComponents encountered. It accounts for nested TransformComponents.
pub fn measure_renderable_subtree_bounds( world: &World, render_assets: &RenderAssets, root: ComponentId, ) -> RenderableBoundsMeasure
pub fn fit_aabb_uniform( aabb: &Aabb, target_bounds: [f32; 6], ) -> Option<UniformFitTransform>