pub struct BvhSystem { /* private fields */ }Implementations
Sourceimpl BvhSystem
impl BvhSystem
pub fn has_index(&self) -> bool
pub fn queue_renderable_added(&mut self, component: ComponentId)
pub fn queue_renderable_removed(&mut self, component: ComponentId)
Sourcepub fn queue_transform_subtree(
&mut self,
world: &World,
transform_root: ComponentId,
)
pub fn queue_transform_subtree( &mut self, world: &World, transform_root: ComponentId, )
Queue all renderables under the given transform subtree for BVH refit.
Sourcepub fn flush_pending(&mut self, world: &World)
pub fn flush_pending(&mut self, world: &World)
Apply any queued add/remove/refit requests.
Intended to be called once after CommandQueue::flush completes.
pub fn raycast_renderables( &self, origin: [f32; 3], dir: [f32; 3], max_distance: f32, ) -> Option<(ComponentId, f32)>
Sourcepub fn raycast_renderables_candidates(
&self,
origin: [f32; 3],
dir: [f32; 3],
max_distance: f32,
limit: usize,
) -> Vec<(ComponentId, f32)>
pub fn raycast_renderables_candidates( &self, origin: [f32; 3], dir: [f32; 3], max_distance: f32, limit: usize, ) -> Vec<(ComponentId, f32)>
Return ray/AABB hit candidates (sorted by ascending t).
This is intended to support narrow-phase hit tests that may reject the closest AABB hit and need to fall through to the next-best candidates.
pub fn query_point(&self, point: [f32; 3]) -> Vec<ComponentId>
pub fn query_aabb(&self, min: [f32; 3], max: [f32; 3]) -> Vec<ComponentId>