pub struct StaticCollisionKey(/* private fields */);Trait Implementations
Sourceimpl Clone for StaticCollisionKey
impl Clone for StaticCollisionKey
Sourcefn clone(&self) -> StaticCollisionKey
fn clone(&self) -> StaticCollisionKey
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreimpl Copy for StaticCollisionKey
Sourceimpl Debug for StaticCollisionKey
impl Debug for StaticCollisionKey
Sourceimpl Default for StaticCollisionKey
impl Default for StaticCollisionKey
Sourcefn default() -> StaticCollisionKey
fn default() -> StaticCollisionKey
Returns the “default value” for a type. Read more
impl Eq for StaticCollisionKey
Sourceimpl From<KeyData> for StaticCollisionKey
impl From<KeyData> for StaticCollisionKey
Sourceimpl Hash for StaticCollisionKey
impl Hash for StaticCollisionKey
Sourceimpl Key for StaticCollisionKey
impl Key for StaticCollisionKey
Sourcefn null() -> Self
fn null() -> Self
Creates a new key that is always invalid and distinct from any non-null
key. A null key can only be created through this method (or default
initialization of keys made with
new_key_type!, which calls this
method). Read moreSourceimpl Ord for StaticCollisionKey
impl Ord for StaticCollisionKey
Sourcefn cmp(&self, other: &StaticCollisionKey) -> Ordering
fn cmp(&self, other: &StaticCollisionKey) -> Ordering
1.21.0 (const: unstable) · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl PartialEq for StaticCollisionKey
impl PartialEq for StaticCollisionKey
Sourceimpl PartialOrd for StaticCollisionKey
impl PartialOrd for StaticCollisionKey
impl StructuralPartialEq for StaticCollisionKey