pub struct GestureSystem {
pub drag_update_policy: DragUpdatePolicy,
/* private fields */
}Fields
drag_update_policy: DragUpdatePolicyImplementations
Sourceimpl GestureSystem
impl GestureSystem
pub fn begin_frame(&mut self)
Sourcepub fn install_handlers(&mut self, rx: &mut RxWorld)
pub fn install_handlers(&mut self, rx: &mut RxWorld)
Install drain-point handlers into RxWorld.
Sourcepub fn state(&self) -> &GestureState
pub fn state(&self) -> &GestureState
Returns the gesture state for the first active pointer, for callers that only care about a single pointer (e.g. editor gizmos, cursor 3D).
pub fn set_drag_update_policy(&mut self, policy: DragUpdatePolicy)
Sourcepub fn tick_with_rx(
&mut self,
visuals: &VisualWorld,
input: &InputState,
activations: &PointerActivations,
pointer_system: &PointerSystem,
rx: &mut RxWorld,
)
pub fn tick_with_rx( &mut self, visuals: &VisualWorld, input: &InputState, activations: &PointerActivations, pointer_system: &PointerSystem, rx: &mut RxWorld, )
Consume RayIntersected signals and PointerActivations to emit DragStart/DragMove/DragEnd/Click.
input is still passed for cursor_pos (screen-space fields on desktop pointer events).
activations drives press/down/release for each pointer regardless of input source.