pub struct TransformGizmoSystem { /* private fields */ }Implementations
Sourceimpl TransformGizmoSystem
impl TransformGizmoSystem
pub fn new() -> Self
pub fn set_editor_context_state( &mut self, editor_context_state: Arc<Mutex<EditorContextState>>, )
Sourcepub fn update_camera_scales(
&mut self,
world: &mut World,
visuals: &VisualWorld,
has_window_camera: bool,
) -> Vec<ComponentId>
pub fn update_camera_scales( &mut self, world: &mut World, visuals: &VisualWorld, has_window_camera: bool, ) -> Vec<ComponentId>
Update the ordinary settings transforms used by camera-specific gizmo anchors. Returns anchors whose effective world matrices must be propagated immediately.
Sourcepub fn install_scoped_handlers_for_gizmo(
&mut self,
rx: &mut RxWorld,
gizmo_root: ComponentId,
)
pub fn install_scoped_handlers_for_gizmo( &mut self, rx: &mut RxWorld, gizmo_root: ComponentId, )
Install per-gizmo scoped handlers rooted at the TransformGizmoComponent node.
Drag events are scoped to the hit renderable; because gizmo handle renderables live under the gizmo node, scoped handlers rooted at the gizmo will run for drag events on its handles.
Sourcepub fn register_transform_gizmo(
&mut self,
world: &mut World,
component: ComponentId,
emit: &mut dyn SignalEmitter,
)
pub fn register_transform_gizmo( &mut self, world: &mut World, component: ComponentId, emit: &mut dyn SignalEmitter, )
Spawn the 9-part gizmo visual subtree for a TransformGizmoComponent.
Contract: TransformGizmoComponent is expected to be attached under a TransformComponent.
pub fn tick_with_queue( &mut self, world: &mut World, _input: &InputState, emit: &mut dyn SignalEmitter, _rx: &mut RxWorld, )