pub struct InputSystem { /* private fields */ }Expand description
System that processes input components and updates transforms based on WASD input.
Intended topology (simple one-way data flow): InputComponent -> TransformComponent -> (Camera2DComponent, RenderableComponent, …)
Implementations
Sourceimpl InputSystem
impl InputSystem
pub fn new() -> Self
Sourcepub fn register_input(&mut self, component: ComponentId)
pub fn register_input(&mut self, component: ComponentId)
Register an InputComponent.
Sourcepub fn process_input(
&mut self,
world: &mut World,
input: &InputState,
emit: &mut dyn SignalEmitter,
dt_sec: f32,
)
pub fn process_input( &mut self, world: &mut World, input: &InputState, emit: &mut dyn SignalEmitter, dt_sec: f32, )
Process input and queue at most one transform update per InputComponent.
This only supports the intended topology: InputComponent -> TransformComponent (child)