pub struct PointerActivations {
pub pressed: Vec<ComponentId>,
pub down: Vec<ComponentId>,
pub released: Vec<ComponentId>,
}Expand description
Which PointerComponent ids have an active trigger state this frame.
Built by PointerSystem::build_activations from InputState (mouse) and XrInputState
(XR controllers). GestureSystem consumes this without knowing the underlying input source.
Fields
pressed: Vec<ComponentId>down: Vec<ComponentId>released: Vec<ComponentId>Trait Implementations
Sourceimpl Clone for PointerActivations
impl Clone for PointerActivations
Sourcefn clone(&self) -> PointerActivations
fn clone(&self) -> PointerActivations
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for PointerActivations
impl Debug for PointerActivations
Sourceimpl Default for PointerActivations
impl Default for PointerActivations
Sourcefn default() -> PointerActivations
fn default() -> PointerActivations
Returns the “default value” for a type. Read more