TextureSystem

Struct TextureSystem 

pub struct TextureSystem { /* private fields */ }

Implementations

Source

impl TextureSystem

pub fn new() -> Self

pub fn register_cached_texture( &mut self, uri: impl Into<String>, handle: TextureHandle, )

Register an already-uploaded texture handle for a URI-like key.

This is intended for virtual keys (e.g. GLTF imported textures like “{gltf_name}:{image_name_or_index}”) where the data is not loaded from disk by TextureSystem.

pub fn register_texture( &mut self, world: &mut World, _visuals: &mut VisualWorld, component: ComponentId, )

pub fn register_texture_filtering( &mut self, world: &mut World, _visuals: &mut VisualWorld, component: ComponentId, )

pub fn flush_pending( &mut self, world: &mut World, visuals: &mut VisualWorld, uploader: &mut dyn TextureUploader, )

Decode+upload any textures that are now attachable to renderables.

Must run after renderables are flushed into VisualWorld so we can update instance handles.