pub struct ReplBackend { /* private fields */ }Expand description
Runs REPL commands against engine state.
This is intended to be called from the main thread (e.g. inside Universe::update()),
after commands are received from the stdin thread.
Implementations
Sourceimpl ReplBackend
impl ReplBackend
pub fn new() -> Self
Sourcepub fn exec(&mut self, world: &World, cmd: &str)
pub fn exec(&mut self, world: &World, cmd: &str)
Execute a single REPL command.
This currently only reads from world and updates internal REPL state (cwd).
Sourcepub fn exec_all<I>(&mut self, world: &World, commands: I)where
I: IntoIterator<Item = String>,
pub fn exec_all<I>(&mut self, world: &World, commands: I)where
I: IntoIterator<Item = String>,
Execute all queued commands.