Windowing

Struct Windowing 

pub struct Windowing;
Minimal winit wrapper (2025 winit style: ApplicationHandler).

Implementations

Source

impl Windowing

pub fn run_app(universe: Universe) -> EngineResult<()>

Run a Universe in a winit window with default input handling.

Examples found in repository?
examples/simple-demo.rs (line 296)
280fn main() {
281    mittens_engine::example_support::ensure_model_assets();
282    utils::logger::init();
283
284    let world = engine::ecs::World::default();
285    let mut universe = engine::Universe::new(world);
286
287    build_demo_scene_7_shapes(&mut universe);
288
289    universe.systems.process_commands(
290        &mut universe.world,
291        &mut universe.visuals,
292        &mut universe.render_assets,
293        &mut universe.command_queue,
294    );
295    universe.enable_repl();
296    engine::Windowing::run_app(universe).expect("Windowing failed");
297}
More examples
Hide additional examples
examples/vr-input-mms.rs (line 29)
3fn main() {
4    mittens_engine::example_support::ensure_model_assets();
5    utils::logger::init();
6
7    let output = scripting::MeowMeowRunner::eval(include_str!("vr-input.mms"));
8
9    for error in &output.errors {
10        eprintln!("[mms] {error}");
11    }
12    println!("[mms] {} intent(s) from vr-input.mms", output.intents.len());
13
14    let world = engine::ecs::World::default();
15    let mut universe = engine::Universe::new(world);
16
17    let scope = engine::ecs::ComponentId::default();
18    for intent in output.intents {
19        universe.command_queue.push_intent_now(scope, intent);
20    }
21
22    universe.systems.process_commands(
23        &mut universe.world,
24        &mut universe.visuals,
25        &mut universe.render_assets,
26        &mut universe.command_queue,
27    );
28
29    engine::Windowing::run_app(universe).expect("Windowing failed");
30}
examples/ui-layout.rs (line 33)
3fn main() {
4    mittens_engine::example_support::ensure_model_assets();
5    utils::logger::init();
6
7    let output = scripting::MeowMeowRunner::eval(include_str!("ui-layout.mms"));
8
9    for error in &output.errors {
10        eprintln!("[mms] {error}");
11    }
12    println!(
13        "[mms] {} intent(s) from ui-layout.mms",
14        output.intents.len()
15    );
16
17    let world = engine::ecs::World::default();
18    let mut universe = engine::Universe::new(world);
19
20    let scope = engine::ecs::ComponentId::default();
21    for intent in output.intents {
22        universe.command_queue.push_intent_now(scope, intent);
23    }
24
25    universe.systems.process_commands(
26        &mut universe.world,
27        &mut universe.visuals,
28        &mut universe.render_assets,
29        &mut universe.command_queue,
30    );
31
32    universe.enable_repl();
33    engine::Windowing::run_app(universe).expect("Windowing failed");
34}
examples/vtuber-desktop.rs (line 33)
3fn main() {
4    mittens_engine::example_support::ensure_model_assets();
5    utils::logger::init();
6
7    let output = scripting::MeowMeowRunner::eval(include_str!("vtuber-desktop.mms"));
8
9    for error in &output.errors {
10        eprintln!("[mms] {error}");
11    }
12    println!(
13        "[mms] {} intent(s) from vtuber-desktop.mms",
14        output.intents.len()
15    );
16
17    let world = engine::ecs::World::default();
18    let mut universe = engine::Universe::new(world);
19
20    let scope = engine::ecs::ComponentId::default();
21    for intent in output.intents {
22        universe.command_queue.push_intent_now(scope, intent);
23    }
24
25    universe.systems.process_commands(
26        &mut universe.world,
27        &mut universe.visuals,
28        &mut universe.render_assets,
29        &mut universe.command_queue,
30    );
31
32    universe.enable_repl();
33    engine::Windowing::run_app(universe).expect("Windowing failed");
34}
examples/inline-block-layout.rs (line 33)
3fn main() {
4    mittens_engine::example_support::ensure_model_assets();
5    utils::logger::init();
6
7    let output = scripting::MeowMeowRunner::eval(include_str!("inline-block-layout.mms"));
8
9    for error in &output.errors {
10        eprintln!("[mms] {error}");
11    }
12    println!(
13        "[mms] {} intent(s) from inline-block-layout.mms",
14        output.intents.len()
15    );
16
17    let world = engine::ecs::World::default();
18    let mut universe = engine::Universe::new(world);
19
20    let scope = engine::ecs::ComponentId::default();
21    for intent in output.intents {
22        universe.command_queue.push_intent_now(scope, intent);
23    }
24
25    universe.systems.process_commands(
26        &mut universe.world,
27        &mut universe.visuals,
28        &mut universe.render_assets,
29        &mut universe.command_queue,
30    );
31
32    universe.enable_repl();
33    engine::Windowing::run_app(universe).expect("Windowing failed");
34}
examples/vtuber-desktop-first-person.rs (line 33)
3fn main() {
4    mittens_engine::example_support::ensure_model_assets();
5    utils::logger::init();
6
7    let output = scripting::MeowMeowRunner::eval(include_str!("vtuber-desktop-first-person.mms"));
8
9    for error in &output.errors {
10        eprintln!("[mms] {error}");
11    }
12    println!(
13        "[mms] {} intent(s) from vtuber-desktop-first-person.mms",
14        output.intents.len()
15    );
16
17    let world = engine::ecs::World::default();
18    let mut universe = engine::Universe::new(world);
19
20    let scope = engine::ecs::ComponentId::default();
21    for intent in output.intents {
22        universe.command_queue.push_intent_now(scope, intent);
23    }
24
25    universe.systems.process_commands(
26        &mut universe.world,
27        &mut universe.visuals,
28        &mut universe.render_assets,
29        &mut universe.command_queue,
30    );
31
32    universe.enable_repl();
33    engine::Windowing::run_app(universe).expect("Windowing failed");
34}
pub fn run_app_with_input( universe: Universe, user_input: UserInput, ) -> EngineResult<()>

Run a Universe in a winit window with a caller-provided UserInput.