EvalResponse

Enum EvalResponse 

pub enum EvalResponse {
    Intent(IntentValue),
    ParsedOk {
        debug_ast: String,
    },
    Error {
        message: String,
    },
    SnippetComplete {
        result: Result<Option<Value>, String>,
    },
    NavigationComplete {
        result: Result<Value, String>,
    },
    ReplReset,
    ShutdownAck,
    HostCall {
        id: u32,
        kind: HostCallKind,
    },
}

Variants



Intent(IntentValue)

A SpawnComponentTree (or other) intent ready to be pushed into the engine.



ParsedOk

Parse-only debug output (from ParseScript).

Fields

debug_ast: String


Error

Fields

message: String


SnippetComplete

Always emitted once for an EvalSnippet, including parse/evaluation failures.

Fields



NavigationComplete

Fields



ReplReset

Tells the engine-side REPL to reset its independently-owned cursor.



ShutdownAck



HostCall

The evaluator needs the host to perform a side-effecting operation and return a result before evaluation can continue. The host must push a matching EvalRequest::HostCallResult { id, value } to unblock the evaluator thread.

Fields

id: u32