pub struct AssetSystem {
pub items: Vec<AssetItem>,
/* private fields */
}Fields
items: Vec<AssetItem>Implementations
Sourceimpl AssetSystem
impl AssetSystem
pub fn new() -> Self
pub fn scan_assets_dir(&mut self, path: &Path) -> Result<(), String>
pub fn load_module(&mut self, path: PathBuf) -> Result<(), String>
pub fn get_item_module(&self, item: &AssetItem) -> Option<&LoadedMmsModule>
pub fn asset_function(&self, item: &AssetItem) -> Option<&Value>
pub fn get_module_name(&self, module_id: AssetModuleId) -> Option<String>
pub fn paint_templates(&self) -> Vec<PaintAssetTemplate>
pub fn build_asset_module_header( &self, world: &mut World, emit: &mut dyn SignalEmitter, module_name: &str, ) -> Result<ComponentId, String>
pub fn spawn_asset_component( &self, item: &AssetItem, args: Vec<Value>, world: &mut World, emit: &mut dyn SignalEmitter, ) -> Result<ComponentId, String>
pub fn spawn_asset_component_uninitialized( &self, item: &AssetItem, args: Vec<Value>, world: &mut World, render_assets: &mut RenderAssets, emit: &mut dyn SignalEmitter, ) -> Result<ComponentId, String>
pub fn spawn_assets_panel( &self, world: &mut World, render_assets: &mut RenderAssets, emit: &mut dyn SignalEmitter, parent: ComponentId, position: (f32, f32, f32), ) -> Result<ComponentId, String>
Sourcepub fn ensure_layout_root_if_needed(
world: &mut World,
root: ComponentId,
) -> Option<ComponentId>
pub fn ensure_layout_root_if_needed( world: &mut World, root: ComponentId, ) -> Option<ComponentId>
If root’s subtree contains styled elements that need layout resolution,
create a LayoutComponent and return its id. Returns None if no layout root is needed.
pub fn build_asset_item_shell( &self, world: &mut World, render_assets: &mut RenderAssets, emit: &mut dyn SignalEmitter, item: &AssetItem, _index: usize, ) -> Result<ComponentId, String>
Sourcepub fn remeasure_pending_previews(
&mut self,
world: &mut World,
render_assets: &RenderAssets,
emit: &mut dyn SignalEmitter,
)
pub fn remeasure_pending_previews( &mut self, world: &mut World, render_assets: &RenderAssets, emit: &mut dyn SignalEmitter, )
After layout has ticked, remeasure any pending preview subtrees that
were spawned with styled content but no RenderableComponent bounds.
Layout-generated background quads now exist, so bounds should return
real AABBs that we can use to compute proper scale/offset.